#if defined (__cplusplus) || defined (c_plusplus)
#include <c_varieties.h>
#ifdef __EXTERN_C__
	EXTERN_FUNCTION ( extern int yylex, ());
#else
	extern int yylex();
#endif
	extern void yyerror(char *);
	extern int yyparse();
#endif

#include <malloc.h>

# line 2 "lua.stx"

char *rcs_luastx = "$Id: lua.stx,v 3.25 1995/10/26 17:02:50 roberto Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mem.h"
#include "opcode.h"
#include "hash.h"
#include "inout.h"
#include "tree.h"
#include "table.h"
#include "lua.h"
#include "func.h"

/* to avoid warnings generated by yacc */
int yyparse (void);
#define malloc luaI_malloc
#define realloc luaI_realloc
#define free luaI_free

#ifndef LISTING
#define LISTING 0
#endif

#ifndef CODE_BLOCK
#define CODE_BLOCK 256
#endif
static int   maxcode;
static int   maxmain;
static Long   maxcurr;  /* to allow maxcurr *= 2 without overflow */
static Byte  *funcCode = NULL;
static Byte **initcode;
static Byte  *basepc;
static int   maincode;
static int   pc;


#define MAXVAR 32
static Long    varbuffer[MAXVAR];    /* variables in an assignment list;
				it's long to store negative Word values */
static int     nvarbuffer=0;	     /* number of variables at a list */

#define MAXLOCALS 32
static Word    localvar[MAXLOCALS];  /* store local variable names */
static int     nlocalvar=0;	     /* number of local variables */

#define MAXFIELDS FIELDS_PER_FLUSH*2
static Word    fields[MAXFIELDS];     /* fieldnames to be flushed */
static int     nfields=0;


/* Internal functions */

static void yyerror (char *s)
{
 static char msg[256];
 sprintf (msg,"%s near \"%s\" at line %d in file `%s'",
          s, lua_lasttext (), lua_linenumber, lua_parsedfile);
 lua_error (msg);
}

static void code_byte (Byte c)
{
 if (pc>maxcurr-2)  /* 1 byte free to code HALT of main code */
 {
  if (maxcurr >= MAX_INT)
    lua_error("code size overflow");
  maxcurr *= 2;
  if (maxcurr >= MAX_INT)
    maxcurr = MAX_INT;
  basepc = growvector(basepc, maxcurr, Byte);
 }
 basepc[pc++] = c;
}

static void code_word (Word n)
{
 CodeWord code;
 code.w = n;
 code_byte(code.m.c1);
 code_byte(code.m.c2);
}

static void code_float (float n)
{
 CodeFloat code;
 code.f = n;
 code_byte(code.m.c1);
 code_byte(code.m.c2);
 code_byte(code.m.c3);
 code_byte(code.m.c4);
}

static void code_code (TFunc *tf)
{
 CodeCode code;
 code.tf = tf;
 code_byte(code.m.c1);
 code_byte(code.m.c2);
 code_byte(code.m.c3);
 code_byte(code.m.c4);
}

static void code_word_at (Byte *p, Word n)
{
 CodeWord code;
 code.w = n;
 *p++ = code.m.c1;
 *p++ = code.m.c2;
}

static void push_field (Word name)
{
  if (nfields < MAXFIELDS)
    fields[nfields++] = name;
  else
   lua_error ("too many fields in nested constructors");
}

static void flush_record (int n)
{
  int i;
  if (n == 0) return;
  code_byte(STORERECORD);
  code_byte(n);
  for (i=0; i<n; i++)
    code_word(fields[--nfields]);
}

static void flush_list (int m, int n)
{
  if (n == 0) return;
  if (m == 0)
    code_byte(STORELIST0); 
  else
  if (m < 255)
  {
    code_byte(STORELIST);
    code_byte(m);
  }
  else
   lua_error ("list constructor too long");
  code_byte(n);
}

static void add_localvar (Word name)
{
 if (nlocalvar < MAXLOCALS)
  localvar[nlocalvar++] = name;
 else
  lua_error ("too many local variables");
}

static void store_localvar (Word name, int n)
{
 if (nlocalvar+n < MAXLOCALS)
  localvar[nlocalvar+n] = name;
 else
  lua_error ("too many local variables");
}

static void add_varbuffer (Long var)
{
 if (nvarbuffer < MAXVAR)
  varbuffer[nvarbuffer++] = var;
 else
  lua_error ("variable buffer overflow");
}

static void code_number (float f)
{ 
  Word i = (Word)f;
  if (f == (float)i)  /* f has an (short) integer value */
  {
   if (i <= 2) code_byte(PUSH0 + i);
   else if (i <= 255)
   {
    code_byte(PUSHBYTE);
    code_byte(i);
   }
   else
   {
    code_byte(PUSHWORD);
    code_word(i);
   }
  }
  else
  {
   code_byte(PUSHFLOAT);
   code_float(f);
  }
}

/*
** Search a local name and if find return its index. If do not find return -1
*/
static int lua_localname (Word n)
{
 int i;
 for (i=nlocalvar-1; i >= 0; i--)
  if (n == localvar[i]) return i;	/* local var */
 return -1;		        /* global var */
}

/*
** Push a variable given a number. If number is positive, push global variable
** indexed by (number -1). If negative, push local indexed by ABS(number)-1.
** Otherwise, if zero, push indexed variable (record).
*/
static void lua_pushvar (Long number)
{ 
 if (number > 0)	/* global var */
 {
  code_byte(PUSHGLOBAL);
  code_word(number-1);
 }
 else if (number < 0)	/* local var */
 {
  number = (-number) - 1;
  if (number < 10) code_byte(PUSHLOCAL0 + number);
  else
  {
   code_byte(PUSHLOCAL);
   code_byte(number);
  }
 }
 else
 {
  code_byte(PUSHINDEXED);
 }
}

static void lua_codeadjust (int n)
{
 if (n+nlocalvar == 0)
   code_byte(ADJUST0);
 else
 {
   code_byte(ADJUST);
   code_byte(n+nlocalvar);
 }
}

static void change2main (void)
{
  /* (re)store main values */
  pc=maincode; basepc=*initcode; maxcurr=maxmain;
  nlocalvar=0;
}

static void savemain (void)
{
  /* save main values */
  maincode=pc; *initcode=basepc; maxmain=maxcurr;
}

static void init_func (void)
{
  if (funcCode == NULL)	/* first function */
  {
   funcCode = newvector(CODE_BLOCK, Byte);
   maxcode = CODE_BLOCK;
  }
  savemain();  /* save main values */
  /* set func values */
  pc=0; basepc=funcCode; maxcurr=maxcode; 
  nlocalvar = 0;
  luaI_codedebugline(lua_linenumber);
}

static void codereturn (void)
{
  if (nlocalvar == 0)
    code_byte(RETCODE0);
  else
  {
    code_byte(RETCODE);
    code_byte(nlocalvar);
  }
}

void luaI_codedebugline (int line)
{
  static int lastline = 0;
  if (lua_debug && line != lastline)
  {
    code_byte(SETLINE);
    code_word(line);
    lastline = line;
  }
}

static int adjust_functioncall (Long exp, int i)
{
  if (exp <= 0)
    return -exp; /* exp is -list length */
  else
  {
    int temp = basepc[exp];
    basepc[exp] = i;
    return temp+i;
  }
}

static void adjust_mult_assign (int vars, Long exps, int temps)
{
  if (exps > 0)
  { /* must correct function call */
    int diff = vars - basepc[exps];
    if (diff >= 0)
      adjust_functioncall(exps, diff);
    else
    {
      adjust_functioncall(exps, 0);
      lua_codeadjust(temps);
    }
  }
  else if (vars != -exps)
    lua_codeadjust(temps);
}

static void storesinglevar (Long v)
{
 if (v > 0)		/* global var */
 {
   code_byte(STOREGLOBAL);
   code_word(v-1);
 }
 else if (v < 0)      /* local var */
 {
   int number = (-v) - 1;
   if (number < 10) code_byte(STORELOCAL0 + number);
   else
   {
     code_byte(STORELOCAL);
     code_byte(number);
   }
 }
 else 
   code_byte(STOREINDEXED0);
}

static void lua_codestore (int i)
{
 if (varbuffer[i] != 0)  /* global or local var */
  storesinglevar(varbuffer[i]);
 else				  /* indexed var */
 {
  int j;
  int upper=0;     	/* number of indexed variables upper */
  int param;		/* number of itens until indexed expression */
  for (j=i+1; j <nvarbuffer; j++)
   if (varbuffer[j] == 0) upper++;
  param = upper*2 + i;
  if (param == 0)
   code_byte(STOREINDEXED0);
  else
  {
   code_byte(STOREINDEXED);
   code_byte(param);
  }
 }
}

static void codeIf (Long thenAdd, Long elseAdd)
{
  Long elseinit = elseAdd+sizeof(Word)+1;
  if (pc == elseinit)		/* no else */
  {
    pc -= sizeof(Word)+1;
    elseinit = pc;
  }
  else
  {
    basepc[elseAdd] = JMP;
    code_word_at(basepc+elseAdd+1, pc-elseinit);
  }
  basepc[thenAdd] = IFFJMP;
  code_word_at(basepc+thenAdd+1,elseinit-(thenAdd+sizeof(Word)+1));
}


/*
** Parse LUA code.
*/
void lua_parse (TFunc *tf)
{
 lua_debug = 0;
 initcode = &(tf->code);
 *initcode = newvector(CODE_BLOCK, Byte);
 maincode = 0; 
 maxmain = CODE_BLOCK;
 change2main();
 if (yyparse ()) lua_error("parse error");
 savemain();
 (*initcode)[maincode++] = RETCODE0;
 tf->size = maincode;
#if LISTING
{ static void PrintCode (Byte *c, Byte *end);
 PrintCode(*initcode,*initcode+maincode); }
#endif
}



# line 411 "lua.stx"
typedef union  
{
 int   vInt;
 float vFloat;
 char *pChar;
 Word  vWord;
 Long  vLong;
 TFunc *pFunc;
 TreeNode *pNode;
} YYSTYPE;
# define WRONGTOKEN 257
# define NIL 258
# define IF 259
# define THEN 260
# define ELSE 261
# define ELSEIF 262
# define WHILE 263
# define DO 264
# define REPEAT 265
# define UNTIL 266
# define END 267
# define RETURN 268
# define LOCAL 269
# define FUNCTION 270
# define NUMBER 271
# define STRING 272
# define NAME 273
# define DEBUG 274
# define AND 275
# define OR 276
# define EQ 277
# define NE 278
# define LE 279
# define GE 280
# define CONC 281
# define UNARY 282
# define NOT 283
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
# define YYERRCODE 256

# line 789 "lua.stx"


#if LISTING

static void PrintCode (Byte *code, Byte *end)
{
 Byte *p = code;
 printf ("\n\nCODE\n");
 while (p != end)
 {
  switch ((OpCode)*p)
  {
   case PUSHNIL:	printf ("%d    PUSHNIL\n", (p++)-code); break;
   case PUSH0: case PUSH1: case PUSH2:
    			printf ("%d    PUSH%c\n", p-code, *p-PUSH0+'0');
    			p++;
   			break;
   case PUSHBYTE:
    			printf ("%d    PUSHBYTE   %d\n", p-code, *(++p));
    			p++;
   			break;
   case PUSHWORD:
                        {
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    PUSHWORD   %d\n", n, c.w);
			}
   			break;
   case PUSHFLOAT:
			{
			 CodeFloat c;
			 int n = p-code;
			 p++;
			 get_float(c,p);
    			 printf ("%d    PUSHFLOAT  %f\n", n, c.f);
			}
   			break;
   case PUSHSTRING:
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    PUSHSTRING   %d\n", n, c.w);
			}
   			break;
   case PUSHFUNCTION:
			{
			 CodeCode c;
			 int n = p-code;
			 p++;
			 get_code(c,p);
			 printf ("%d    PUSHFUNCTION  %p\n", n, c.tf);
			}
			break;

   case PUSHLOCAL0: case PUSHLOCAL1: case PUSHLOCAL2: case PUSHLOCAL3:
   case PUSHLOCAL4: case PUSHLOCAL5: case PUSHLOCAL6: case PUSHLOCAL7:
   case PUSHLOCAL8: case PUSHLOCAL9:
    			printf ("%d    PUSHLOCAL%c\n", p-code, *p-PUSHLOCAL0+'0');
    			p++;
   			break;
   case PUSHLOCAL:	printf ("%d    PUSHLOCAL   %d\n", p-code, *(++p));
    			p++;
   			break;
   case PUSHGLOBAL:
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    PUSHGLOBAL   %d\n", n, c.w);
			}
   			break;
   case PUSHINDEXED:    printf ("%d    PUSHINDEXED\n", (p++)-code); break;
   case STORELOCAL0: case STORELOCAL1: case STORELOCAL2: case STORELOCAL3:
   case STORELOCAL4: case STORELOCAL5: case STORELOCAL6: case STORELOCAL7:
   case STORELOCAL8: case STORELOCAL9:
    			printf ("%d    STORELOCAL%c\n", p-code, *p-STORELOCAL0+'0');
    			p++;
   			break;
   case STORELOCAL:
    			printf ("%d    STORELOCAL   %d\n", p-code, *(++p));
    			p++;
   			break;
   case STOREGLOBAL:
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    STOREGLOBAL   %d\n", n, c.w);
			}
   			break;
   case PUSHSELF:
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    PUSHSELF   %d\n", n, c.w);
			}
			break;
   case STOREINDEXED0:  printf ("%d    STOREINDEXED0\n", (p++)-code); break;
   case STOREINDEXED:   printf ("%d    STOREINDEXED   %d\n", p-code, *(++p));
    			p++;
   			break;
   case STORELIST0:
	printf("%d      STORELIST0  %d\n", p-code, *(++p));
        p++;
        break;
   case STORELIST:
	printf("%d      STORELIST  %d %d\n", p-code, *(p+1), *(p+2));
        p+=3;
        break;
   case STORERECORD:
       printf("%d      STORERECORD  %d\n", p-code, *(++p));
       p += *p*sizeof(Word) + 1;
       break;
   case ADJUST0: printf ("%d    ADJUST0\n", (p++)-code); break;
   case ADJUST:
    			printf ("%d    ADJUST   %d\n", p-code, *(++p));
    			p++;
   			break;
   case CREATEARRAY:	
    {
      CodeWord c;
      int n = p-code;
      p++;
      get_word(c,p);
      printf ("%d    CREATEARRAY %d\n", n, c.w);
      break;
    }
   case EQOP:       	printf ("%d    EQOP\n", (p++)-code); break;
   case LTOP:       	printf ("%d    LTOP\n", (p++)-code); break;
   case LEOP:       	printf ("%d    LEOP\n", (p++)-code); break;
   case ADDOP:       	printf ("%d    ADDOP\n", (p++)-code); break;
   case SUBOP:       	printf ("%d    SUBOP\n", (p++)-code); break;
   case MULTOP:      	printf ("%d    MULTOP\n", (p++)-code); break;
   case DIVOP:       	printf ("%d    DIVOP\n", (p++)-code); break;
   case POWOP:       	printf ("%d    POWOP\n", (p++)-code); break;
   case CONCOP:       	printf ("%d    CONCOP\n", (p++)-code); break;
   case MINUSOP:       	printf ("%d    MINUSOP\n", (p++)-code); break;
   case NOTOP:       	printf ("%d    NOTOP\n", (p++)-code); break;
   case ONTJMP:	   
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    ONTJMP  %d\n", n, c.w);
			}
   			break;
   case ONFJMP:	   
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    ONFJMP  %d\n", n, c.w);
			}
   			break;
   case JMP:	   
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    JMP  %d\n", n, c.w);
			}
   			break;
   case UPJMP:
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    UPJMP  %d\n", n, c.w);
			}
   			break;
   case IFFJMP:
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    IFFJMP  %d\n", n, c.w);
			}
   			break;
   case IFFUPJMP:
    			{
			 CodeWord c;
			 int n = p-code;
			 p++;
			 get_word(c,p);
    			 printf ("%d    IFFUPJMP  %d\n", n, c.w);
			}
   			break;
   case POP:       	printf ("%d    POP\n", (p++)-code); break;
   case CALLFUNC:
	printf ("%d    CALLFUNC %d %d\n", p-code, *(p+1), *(p+2));
	p+=3;
	break;
   case RETCODE0: printf ("%d    RETCODE0\n", (p++)-code); break;
   case RETCODE:
    			printf ("%d    RETCODE   %d\n", p-code, *(++p));
    			p++;
   			break;
   case SETLINE:
                        {
                         CodeWord c;
                         int n = p-code;
                         p++;
                         get_word(c,p);
                         printf ("%d    SETLINE  %d\n", n, c.w);
                        }
                        break;

   default:		printf ("%d    Cannot happen: code %d\n", (p++)-code, *(p-1)); break;
  }
 }
}
#endif

int yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
-1, 14,
	61, 88,
	44, 88,
	-2, 94,
-1, 22,
	40, 7,
	-2, 94,
-1, 29,
	40, 59,
	123, 59,
	-2, 46,
-1, 44,
	123, 56,
	-2, 63,
-1, 71,
	123, 56,
	-2, 84,
-1, 76,
	275, 30,
	276, 30,
	277, 30,
	278, 30,
	62, 30,
	60, 30,
	279, 30,
	280, 30,
	281, 30,
	43, 30,
	45, 30,
	42, 30,
	47, 30,
	94, 30,
	-2, 65,
-1, 77,
	91, 94,
	46, 94,
	-2, 89,
-1, 132,
	123, 56,
	-2, 78,
-1, 138,
	123, 56,
	-2, 63,
-1, 155,
	275, 30,
	276, 30,
	277, 30,
	278, 30,
	62, 30,
	60, 30,
	279, 30,
	280, 30,
	281, 30,
	43, 30,
	45, 30,
	42, 30,
	47, 30,
	94, 30,
	-2, 67,
	};
# define YYNPROD 100
# define YYLAST 311
int yyact[]={

    61,    59,   148,    60,   141,    62,   118,    61,    59,    90,
    60,    89,    62,    86,    84,    18,    42,   168,    54,   164,
    55,    61,    59,   156,    60,    54,    62,    55,    61,    59,
   115,    60,    73,    62,   157,    61,    59,    19,    60,    54,
    62,    55,    61,    59,    82,    60,    54,    62,    55,   158,
   159,   129,    63,    54,    91,    55,   111,    25,   121,    63,
    54,   109,    55,   127,    26,    61,    59,    26,    60,    27,
    62,     7,    27,    63,    71,     8,    33,     9,    11,    63,
    63,    12,     6,    80,    67,    18,    13,    63,    68,     7,
    48,    40,     4,     8,    63,     9,    24,    76,   138,    12,
    81,   133,    76,    18,    61,    59,    61,    60,    39,    62,
    20,    62,   146,   130,   117,   132,    69,    63,   123,    48,
   104,   105,   122,    70,   124,   120,    72,   106,    48,    50,
    29,    46,    17,    44,   128,    47,    85,    23,    83,    76,
    51,    28,    92,    93,    94,    95,    96,    97,    98,    99,
   100,   101,   102,   103,    88,   140,    63,    45,    63,    36,
   112,    14,   131,   139,    47,    35,    22,   151,   126,   134,
   125,    78,   137,    47,   153,    74,    38,    37,    75,   142,
   116,     5,     3,   154,     2,    49,    21,   147,    16,    87,
   152,   165,   163,    11,   110,   108,    76,   155,   145,   160,
    77,    79,    41,   171,   135,   107,   162,   173,   161,   136,
    15,    43,    10,   167,   143,   144,     1,     0,   169,     0,
   119,   149,   150,     0,   170,     0,   172,     0,     0,     0,
     0,     0,     0,    65,    66,    53,    56,    57,    58,    64,
    65,    66,    53,    56,    57,    58,    64,    17,   166,     0,
   114,     0,     0,    52,    65,    66,    53,    56,    57,    58,
    64,    65,    66,    53,    56,    57,    58,    64,    65,    66,
    53,    56,    57,    58,    64,     0,    14,    53,    56,    57,
    58,    64,    32,     0,     0,    32,     0,     0,     0,     0,
     0,     0,     0,     0,     0,    30,    31,    18,    30,    31,
   113,     0,     0,     0,    64,     0,     0,    34,     0,     0,
    34 };
int yypact[]={

 -1000,  -188, -1000, -1000,    51, -1000,  -258,    24, -1000, -1000,
    47, -1000,  -257, -1000, -1000,    93, -1000,    73, -1000, -1000,
 -1000,    89, -1000,    82,    -7, -1000,    24,    24, -1000,    73,
 -1000, -1000, -1000, -1000,    24,   -49, -1000,    24, -1000,    24,
  -258,    39, -1000, -1000,    24, -1000,  -259,    24,  -260, -1000,
  -262,  -264, -1000,    24,    24,    24,    24,    24,    24,    24,
    24,    24,    24,    24,    24, -1000, -1000,    86,   -21,   -15,
   -15,    27,   -14,  -236, -1000,    70, -1000, -1000,    44, -1000,
  -267,    24,    84,    70, -1000,   -35, -1000,    81,    74, -1000,
 -1000, -1000,    23,    23,    23,    23,    23,    23,    64,    64,
   -15,   -15,   -15,    62, -1000, -1000, -1000,   -62, -1000,    69,
    71, -1000,   -21,    40, -1000,    24,  -170, -1000, -1000,    70,
 -1000, -1000, -1000,  -269, -1000,    24,    24, -1000,    53, -1000,
  -271, -1000,    24,    24, -1000,   -21,    51, -1000,    24,    24,
  -244, -1000,  -212,     0,     0, -1000,  -271, -1000,    40,   -21,
   -21, -1000, -1000, -1000,    51, -1000, -1000,  -248, -1000,    24,
 -1000,    69,  -250, -1000, -1000, -1000,   -42, -1000, -1000, -1000,
 -1000, -1000,  -212, -1000 };
int yypgo[]={

     0,   216,    54,    44,   138,    76,    57,   212,   211,   210,
   205,   202,   201,   199,    61,   198,   195,   194,   189,   159,
   188,   186,   185,   184,   182,    92,    37,   181,   130,    32,
   180,    88,    34,   177,   176,   175,   172,   141,   170,   168,
   165,   163,   154,   134,    51,    56 };
int yyr1[]={

     0,     1,     1,     1,    23,    23,    24,    21,    21,    22,
    30,    30,    26,    26,    25,    33,    25,    34,    25,    25,
    25,    25,    32,    32,    32,    35,    29,    36,    36,     2,
    31,     6,     6,     6,     6,     6,     6,     6,     6,     6,
     6,     6,     6,     6,     6,     6,     6,     6,     6,     6,
     6,     6,    38,     6,    39,     6,    40,    37,     5,     9,
     9,     8,     8,     3,     3,     4,    41,     4,    18,    18,
    42,    42,    43,    10,    10,    15,    15,    44,    44,    13,
    13,    14,    14,    45,    16,    16,    17,    17,     7,     7,
    19,    19,    19,    20,    28,    11,    11,    12,    12,    27 };
int yyr2[]={

     0,     0,     4,     4,     4,     2,     7,     3,     7,    11,
     0,     6,     0,     2,    17,     1,    17,     1,    13,     7,
     2,     7,     0,     4,    15,     1,     7,     0,     7,     1,
     3,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     5,     3,     3,     3,     3,     3,
     3,     5,     1,    11,     1,    11,     1,     9,     5,     3,
     7,     7,     3,     1,     3,     3,     1,     9,     1,     3,
     3,     7,     1,     7,     5,     1,     5,     0,     2,     1,
     5,     3,     7,     7,     1,     5,     3,     7,     3,     7,
     3,     9,     7,     3,     3,     3,     7,     1,     5,     3 };
int yychk[]={

 -1000,    -1,   -23,   -24,   -25,   -27,   270,   259,   263,   265,
    -7,    -5,   269,   274,   -19,    -9,   -20,   -28,   273,   -26,
    59,   -21,   -19,   -28,   -31,    -6,    40,    45,   -37,   -28,
   271,   272,   258,    -5,   283,   -40,   -19,   -33,   -34,    61,
    44,   -11,   273,    -8,    40,   -37,    58,    91,    46,   -22,
    40,    58,   260,   277,    60,    62,   278,   279,   280,    43,
    45,    42,    47,    94,   281,   275,   276,    -6,   -31,   -31,
   -31,   123,   -31,   -29,   -35,    -4,    -6,   -19,   -28,   -12,
    44,    61,    -3,    -4,   273,   -31,   273,   -18,   -42,   273,
   273,    -2,   -31,   -31,   -31,   -31,   -31,   -31,   -31,   -31,
   -31,   -31,   -31,   -31,    -2,    -2,    41,   -10,   -16,   -14,
   -17,   -45,   -31,   273,   264,   266,   -30,    44,   273,    -4,
    41,    93,    41,    44,   -29,   -38,   -39,   125,   -43,   -44,
    44,   -44,    44,    61,    -2,   -31,   -25,   -36,   268,   -41,
   -29,   273,    -2,   -31,   -31,   -15,    59,   -45,   273,   -31,
   -31,   -29,    -2,   -26,    -3,    -6,   267,   -32,   261,   262,
   -13,   -14,    -2,   -26,   267,   -29,   -31,   -44,   267,   260,
    -2,   -29,    -2,   -32 };
int yydef[]={

     1,    -2,     2,     3,    12,     5,     0,    56,    15,    17,
     0,    20,     0,    99,    -2,    56,    90,    59,    93,     4,
    13,     0,    -2,     0,     0,    30,    56,    56,    45,    -2,
    47,    48,    49,    50,    56,     0,    94,    56,    25,    56,
     0,    97,    95,    58,    -2,    62,     0,    56,     0,     6,
    68,     0,    29,    56,    56,    56,    56,    56,    56,    56,
    56,    56,    56,    56,    56,    29,    29,    30,     0,    44,
    51,    -2,     0,     0,    10,    19,    -2,    -2,     0,    21,
     0,    56,     0,    64,    60,     0,    92,     0,    69,    70,
     8,    25,    32,    33,    34,    35,    36,    37,    38,    39,
    40,    41,    42,    43,    52,    54,    31,     0,    72,    77,
    77,    81,    86,    93,    29,    56,    27,    66,    96,    98,
    61,    91,    25,     0,    29,    56,    56,    57,    75,    74,
    78,    85,    -2,    56,    25,    29,    12,    26,    -2,    56,
     0,    71,    22,    53,    55,    73,    79,    82,     0,    87,
    83,    29,    18,    11,    12,    -2,     9,     0,    25,    56,
    76,    77,     0,    28,    14,    23,     0,    80,    16,    29,
    25,    29,    22,    24 };
typedef struct { char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

yytoktype yytoks[] =
{
	"WRONGTOKEN",	257,
	"NIL",	258,
	"IF",	259,
	"THEN",	260,
	"ELSE",	261,
	"ELSEIF",	262,
	"WHILE",	263,
	"DO",	264,
	"REPEAT",	265,
	"UNTIL",	266,
	"END",	267,
	"RETURN",	268,
	"LOCAL",	269,
	"FUNCTION",	270,
	"NUMBER",	271,
	"STRING",	272,
	"NAME",	273,
	"DEBUG",	274,
	"AND",	275,
	"OR",	276,
	"EQ",	277,
	"NE",	278,
	">",	62,
	"<",	60,
	"LE",	279,
	"GE",	280,
	"CONC",	281,
	"+",	43,
	"-",	45,
	"*",	42,
	"/",	47,
	"UNARY",	282,
	"NOT",	283,
	"^",	94,
	"-unknown-",	-1	/* ends search */
};

char * yyreds[] =
{
	"-no such reduction-",
	"functionlist : /* empty */",
	"functionlist : functionlist globalstat",
	"functionlist : functionlist function",
	"globalstat : stat sc",
	"globalstat : setdebug",
	"function : FUNCTION funcname body",
	"funcname : var",
	"funcname : varexp ':' NAME",
	"body : '(' parlist ')' block END",
	"statlist : /* empty */",
	"statlist : statlist stat sc",
	"sc : /* empty */",
	"sc : ';'",
	"stat : IF expr1 THEN PrepJump block PrepJump elsepart END",
	"stat : WHILE",
	"stat : WHILE expr1 DO PrepJump block PrepJump END",
	"stat : REPEAT",
	"stat : REPEAT block UNTIL expr1 PrepJump",
	"stat : varlist1 '=' exprlist1",
	"stat : functioncall",
	"stat : LOCAL localdeclist decinit",
	"elsepart : /* empty */",
	"elsepart : ELSE block",
	"elsepart : ELSEIF expr1 THEN PrepJump block PrepJump elsepart",
	"block : /* empty */",
	"block : statlist ret",
	"ret : /* empty */",
	"ret : RETURN exprlist sc",
	"PrepJump : /* empty */",
	"expr1 : expr",
	"expr : '(' expr ')'",
	"expr : expr1 EQ expr1",
	"expr : expr1 '<' expr1",
	"expr : expr1 '>' expr1",
	"expr : expr1 NE expr1",
	"expr : expr1 LE expr1",
	"expr : expr1 GE expr1",
	"expr : expr1 '+' expr1",
	"expr : expr1 '-' expr1",
	"expr : expr1 '*' expr1",
	"expr : expr1 '/' expr1",
	"expr : expr1 '^' expr1",
	"expr : expr1 CONC expr1",
	"expr : '-' expr1",
	"expr : table",
	"expr : varexp",
	"expr : NUMBER",
	"expr : STRING",
	"expr : NIL",
	"expr : functioncall",
	"expr : NOT expr1",
	"expr : expr1 AND PrepJump",
	"expr : expr1 AND PrepJump expr1",
	"expr : expr1 OR PrepJump",
	"expr : expr1 OR PrepJump expr1",
	"table : /* empty */",
	"table : '{' fieldlist '}'",
	"functioncall : funcvalue funcParams",
	"funcvalue : varexp",
	"funcvalue : varexp ':' NAME",
	"funcParams : '(' exprlist ')'",
	"funcParams : table",
	"exprlist : /* empty */",
	"exprlist : exprlist1",
	"exprlist1 : expr",
	"exprlist1 : exprlist1 ','",
	"exprlist1 : exprlist1 ',' expr",
	"parlist : /* empty */",
	"parlist : parlist1",
	"parlist1 : NAME",
	"parlist1 : parlist1 ',' NAME",
	"fieldlist : lfieldlist",
	"fieldlist : lfieldlist semicolonpart",
	"fieldlist : ffieldlist1 lastcomma",
	"semicolonpart : /* empty */",
	"semicolonpart : ';' ffieldlist",
	"lastcomma : /* empty */",
	"lastcomma : ','",
	"ffieldlist : /* empty */",
	"ffieldlist : ffieldlist1 lastcomma",
	"ffieldlist1 : ffield",
	"ffieldlist1 : ffieldlist1 ',' ffield",
	"ffield : NAME '=' expr1",
	"lfieldlist : /* empty */",
	"lfieldlist : lfieldlist1 lastcomma",
	"lfieldlist1 : expr1",
	"lfieldlist1 : lfieldlist1 ',' expr1",
	"varlist1 : var",
	"varlist1 : varlist1 ',' var",
	"var : singlevar",
	"var : varexp '[' expr1 ']'",
	"var : varexp '.' NAME",
	"singlevar : NAME",
	"varexp : var",
	"localdeclist : NAME",
	"localdeclist : localdeclist ',' NAME",
	"decinit : /* empty */",
	"decinit : '=' exprlist1",
	"setdebug : DEBUG",
};
#endif /* YYDEBUG */
#line 1 "/usr/lang/SC1.0/yaccpar"
/*	@(#)yaccpar 1.10 89/04/04 SMI; from S5R3 1.10	*/

/*
** Skeleton parser driver for yacc output
*/
/*
	@(#)RELEASE SC1.0 C++ 2.1 1Mar1991
*/

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( "syntax error - cannot backup" );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#define YYCOPY(to, from, type) \
	(type *) memcpy(to, (char *) from, yynewmax * sizeof(type))
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif


/*
** extern declarations for C++ - check your own code for correctness
** if you have function redefined error messages here.
*/

#ifdef __cplusplus
EXTERN_FUNCTION ( extern int printf,    (const char*, DOTDOTDOT) );
EXTERN_FUNCTION ( extern void *memcpy,  (void *, const void *, int) );
#endif


/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-1000)

/*
** static variables used by the parser
*/
static YYSTYPE yy_yyv[YYMAXDEPTH], *yyv = yy_yyv;	/* value stack */
static int yy_yys[YYMAXDEPTH], *yys = yy_yys;	/* state stack */

static YYSTYPE *yypv;			/* top of value stack */
static int *yyps;			/* top of state stack */

static int yystate;			/* current state */
static int yytmp;			/* extra var (lasts between blocks) */

#if defined(__cplusplus) || defined(__STDC__) || defined(lint)
static int __yaccpar_lint_hack__ = 0;   
				/* if you change the value from 0 to
				something else, make sure you know
				what to do with yyerrlab reference.
				This is a hack - to make sure C++ and
				lint are happy with the 4.1 yacc code. */
#endif

int yynerrs;			/* number of errors */

int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */
static unsigned yymaxdepth = YYMAXDEPTH;


/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/
int
yyparse()
{
        register YYSTYPE *yypvt = (YYSTYPE*)0 ;        /* top of value stack for
$vars */

	/*
	** Initialize externals - yyparse may be called more than once
	*/
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

#if defined(__cplusplus) || defined(__STDC__) || defined(lint)
/*
   Note that the following can never be executed but simply to please
   lint and C++
 */
	switch (__yaccpar_lint_hack__)
	{
		case 1: goto yyerrlab;
		case 2: goto yynewstate;
	}
#endif

	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */

	goto yystack;

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			(void)printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				(void)printf( "end-of-file\n" );
			else if ( yychar < 0 )
				(void)printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				(void)printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= &yys[ yymaxdepth ] )	/* room on stack? */
		{
                        /*
                        ** reallocate and recover.  Note that pointers
                        ** have to be reset, or bad things will happen
                        */ 
                        int yyps_index = (yy_ps - yys);
                        int yypv_index = (yy_pv - yyv);
                        int yypvt_index = (yypvt - yyv);
                        int yynewmax;

			yynewmax = yymaxdepth + YYMAXDEPTH;
			if (yymaxdepth == YYMAXDEPTH)	/* first time growth */
			{
				YYSTYPE *newyyv = (YYSTYPE*)malloc(yynewmax*sizeof(YYSTYPE));
				int *newyys = (int*)malloc(yynewmax*sizeof(int));
				if (newyys != 0 && newyyv != 0)
				{
					yys = YYCOPY(newyys, yys, int);
					yyv = YYCOPY(newyyv, yyv, YYSTYPE);
				}
				else
					yynewmax = 0;	/* failed */
			}
			else				/* not first time */
			{
				yyv = (YYSTYPE*)realloc((char*)yyv,
                                	yynewmax * sizeof(YYSTYPE));
				yys = (int*)realloc((char*)yys,
					yynewmax * sizeof(int));
				if (yys == 0 || yyv == 0)
					yynewmax = 0;	/* failed */
			}
			if (yynewmax <= yymaxdepth)	/* tables not expanded */
			{
				yyerror( "yacc stack overflow" );
				YYABORT;
			}
			yymaxdepth = yynewmax;

                        yy_ps = yys + yyps_index;
                        yy_pv = yyv + yypv_index;
                        yypvt = yyv + yypvt_index;
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			(void)printf( "Received token " );
			if ( yychar == 0 )
				(void)printf( "end-of-file\n" );
			else if ( yychar < 0 )
				(void)printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				(void)printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				(void)printf( "Received token " );
				if ( yychar == 0 )
					(void)printf( "end-of-file\n" );
				else if ( yychar < 0 )
					(void)printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					(void)printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register int *yyxi = yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( "syntax error" );
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
				yynerrs++;
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						(void)printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					(void)printf( "Error recovery discards " );
					if ( yychar == 0 )
						(void)printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						(void)printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						(void)printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			(void)printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
                yypvt = yy_pv;                  /* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 6:
# line 470 "lua.stx"
{ 
		code_byte(PUSHFUNCTION);
		code_code(yypvt[-0].pFunc);
		storesinglevar(yypvt[-1].vLong);
	       } break;
case 7:
# line 477 "lua.stx"
{ yyval.vLong =yypvt[-0].vLong; init_func(); } break;
case 8:
# line 479 "lua.stx"
{
	  code_byte(PUSHSTRING);
	  code_word(luaI_findconstant(yypvt[-0].pNode));
	  yyval.vLong = 0;  /* indexed variable */
	  init_func();
	  add_localvar(luaI_findsymbolbyname("self"));
	} break;
case 9:
# line 489 "lua.stx"
{
          codereturn();
	  yyval.pFunc = new(TFunc);
	  yyval.pFunc->size = pc;
	  yyval.pFunc->code = newvector(pc, Byte);
	  yyval.pFunc->fileName = lua_parsedfile;
	  yyval.pFunc->lineDefined = yypvt[-3].vInt;
	  memcpy(yyval.pFunc->code, basepc, pc*sizeof(Byte));
	  /* save func values */
	  funcCode = basepc; maxcode=maxcurr;
#if LISTING
                PrintCode(funcCode,funcCode+pc);
#endif
	  change2main();  /* change back to main code */
	} break;
case 14:
# line 513 "lua.stx"
{ codeIf(yypvt[-4].vLong, yypvt[-2].vLong); } break;
case 15:
# line 515 "lua.stx"
{yyval.vLong=pc;} break;
case 16:
# line 516 "lua.stx"
{
        basepc[yypvt[-3].vLong] = IFFJMP;
	code_word_at(basepc+yypvt[-3].vLong+1, pc - (yypvt[-3].vLong + sizeof(Word)+1));
        basepc[yypvt[-1].vLong] = UPJMP;
	code_word_at(basepc+yypvt[-1].vLong+1, pc - (yypvt[-6].vLong));
       } break;
case 17:
# line 523 "lua.stx"
{yyval.vLong=pc;} break;
case 18:
# line 524 "lua.stx"
{
        basepc[yypvt[-0].vLong] = IFFUPJMP;
	code_word_at(basepc+yypvt[-0].vLong+1, pc - (yypvt[-4].vLong));
       } break;
case 19:
# line 530 "lua.stx"
{
        {
         int i;
         adjust_mult_assign(nvarbuffer, yypvt[-0].vLong, yypvt[-2].vInt * 2 + nvarbuffer);
	 for (i=nvarbuffer-1; i>=0; i--)
	  lua_codestore (i);
	 if (yypvt[-2].vInt > 1 || (yypvt[-2].vInt == 1 && varbuffer[0] != 0))
	  lua_codeadjust (0);
	}
       } break;
case 21:
# line 542 "lua.stx"
{ nlocalvar += yypvt[-1].vInt;
	  adjust_mult_assign(yypvt[-1].vInt, yypvt[-0].vInt, 0);
	} break;
case 24:
# line 550 "lua.stx"
{ codeIf(yypvt[-3].vLong, yypvt[-1].vLong); } break;
case 25:
# line 553 "lua.stx"
{yyval.vInt = nlocalvar;} break;
case 26:
# line 554 "lua.stx"
{
	  if (nlocalvar != yypvt[-2].vInt)
	  {
           nlocalvar = yypvt[-2].vInt;
	   lua_codeadjust (0);
	  }
         } break;
case 28:
# line 565 "lua.stx"
{
	   adjust_functioncall(yypvt[-1].vLong, MULT_RET);
           codereturn();
          } break;
case 29:
# line 572 "lua.stx"
{ 
	  yyval.vLong = pc;
	  code_byte(0);		/* open space */
	  code_word (0);
         } break;
case 30:
# line 578 "lua.stx"
{ adjust_functioncall(yypvt[-0].vLong, 1); } break;
case 31:
# line 581 "lua.stx"
{ yyval.vLong = yypvt[-1].vLong; } break;
case 32:
# line 582 "lua.stx"
{ code_byte(EQOP);   yyval.vLong = 0; } break;
case 33:
# line 583 "lua.stx"
{ code_byte(LTOP);   yyval.vLong = 0; } break;
case 34:
# line 584 "lua.stx"
{ code_byte(GTOP);   yyval.vLong = 0; } break;
case 35:
# line 585 "lua.stx"
{ code_byte(EQOP); code_byte(NOTOP); yyval.vLong = 0; } break;
case 36:
# line 586 "lua.stx"
{ code_byte(LEOP);   yyval.vLong = 0; } break;
case 37:
# line 587 "lua.stx"
{ code_byte(GEOP);   yyval.vLong = 0; } break;
case 38:
# line 588 "lua.stx"
{ code_byte(ADDOP);  yyval.vLong = 0; } break;
case 39:
# line 589 "lua.stx"
{ code_byte(SUBOP);  yyval.vLong = 0; } break;
case 40:
# line 590 "lua.stx"
{ code_byte(MULTOP); yyval.vLong = 0; } break;
case 41:
# line 591 "lua.stx"
{ code_byte(DIVOP);  yyval.vLong = 0; } break;
case 42:
# line 592 "lua.stx"
{ code_byte(POWOP);  yyval.vLong = 0; } break;
case 43:
# line 593 "lua.stx"
{ code_byte(CONCOP);  yyval.vLong = 0; } break;
case 44:
# line 594 "lua.stx"
{ code_byte(MINUSOP); yyval.vLong = 0;} break;
case 45:
# line 595 "lua.stx"
{ yyval.vLong = 0; } break;
case 46:
# line 596 "lua.stx"
{ yyval.vLong = 0;} break;
case 47:
# line 597 "lua.stx"
{ code_number(yypvt[-0].vFloat); yyval.vLong = 0; } break;
case 48:
# line 599 "lua.stx"
{
      code_byte(PUSHSTRING);
      code_word(yypvt[-0].vWord);
      yyval.vLong = 0;
     } break;
case 49:
# line 604 "lua.stx"
{code_byte(PUSHNIL); yyval.vLong = 0; } break;
case 50:
# line 605 "lua.stx"
{ yyval.vLong = yypvt[-0].vLong; } break;
case 51:
# line 606 "lua.stx"
{ code_byte(NOTOP);  yyval.vLong = 0;} break;
case 52:
# line 607 "lua.stx"
{code_byte(POP); } break;
case 53:
# line 608 "lua.stx"
{ 
      basepc[yypvt[-2].vLong] = ONFJMP;
      code_word_at(basepc+yypvt[-2].vLong+1, pc - (yypvt[-2].vLong + sizeof(Word)+1));
      yyval.vLong = 0;
     } break;
case 54:
# line 613 "lua.stx"
{code_byte(POP); } break;
case 55:
# line 614 "lua.stx"
{ 
      basepc[yypvt[-2].vLong] = ONTJMP;
      code_word_at(basepc+yypvt[-2].vLong+1, pc - (yypvt[-2].vLong + sizeof(Word)+1));
      yyval.vLong = 0;
     } break;
case 56:
# line 622 "lua.stx"
{
      code_byte(CREATEARRAY);
      yyval.vLong = pc; code_word(0);
     } break;
case 57:
# line 627 "lua.stx"
{
      code_word_at(basepc+yypvt[-3].vLong, yypvt[-1].vInt);
     } break;
case 58:
# line 633 "lua.stx"
{
	  code_byte(CALLFUNC);
	  code_byte(yypvt[-1].vInt+yypvt[-0].vInt);
	  yyval.vLong = pc;
	  code_byte(0);  /* may be modified by other rules */
	} break;
case 59:
# line 641 "lua.stx"
{ yyval.vInt = 0; } break;
case 60:
# line 643 "lua.stx"
{ 
               code_byte(PUSHSELF); 
	       code_word(luaI_findconstant(yypvt[-0].pNode));
               yyval.vInt = 1;
	     } break;
case 61:
# line 651 "lua.stx"
{ yyval.vInt = adjust_functioncall(yypvt[-1].vLong, 1); } break;
case 62:
# line 652 "lua.stx"
{ yyval.vInt = 1; } break;
case 63:
# line 655 "lua.stx"
{ yyval.vLong = 0; } break;
case 64:
# line 656 "lua.stx"
{ yyval.vLong = yypvt[-0].vLong; } break;
case 65:
# line 659 "lua.stx"
{ if (yypvt[-0].vLong != 0) yyval.vLong = yypvt[-0].vLong; else yyval.vLong = -1; } break;
case 66:
# line 660 "lua.stx"
{ yyval.vLong = adjust_functioncall(yypvt[-1].vLong, 1); } break;
case 67:
# line 661 "lua.stx"
{
	  if (yypvt[-0].vLong == 0) yyval.vLong = -(yypvt[-1].vLong + 1);  /* -length */
	  else
	  {
	    adjust_functioncall(yypvt[-0].vLong, yypvt[-1].vLong);
	    yyval.vLong = yypvt[-0].vLong;
	  }
	} break;
case 68:
# line 671 "lua.stx"
{ lua_codeadjust(0); yyval.vInt = lua_linenumber; } break;
case 69:
# line 672 "lua.stx"
{ lua_codeadjust(0); yyval.vInt = lua_linenumber; } break;
case 70:
# line 676 "lua.stx"
{
		 add_localvar(luaI_findsymbol(yypvt[-0].pNode));
		} break;
case 71:
# line 680 "lua.stx"
{
		 add_localvar(luaI_findsymbol(yypvt[-0].pNode));
		} break;
case 72:
# line 686 "lua.stx"
{ flush_list(yypvt[-0].vInt/FIELDS_PER_FLUSH, yypvt[-0].vInt%FIELDS_PER_FLUSH); } break;
case 73:
# line 688 "lua.stx"
{ yyval.vInt = yypvt[-2].vInt+yypvt[-0].vInt; } break;
case 74:
# line 690 "lua.stx"
{ yyval.vInt = yypvt[-1].vInt; flush_record(yypvt[-1].vInt%FIELDS_PER_FLUSH); } break;
case 75:
# line 694 "lua.stx"
{ yyval.vInt = 0; } break;
case 76:
# line 696 "lua.stx"
{ yyval.vInt = yypvt[-0].vInt; flush_record(yypvt[-0].vInt%FIELDS_PER_FLUSH); } break;
case 79:
# line 703 "lua.stx"
{ yyval.vInt = 0; } break;
case 80:
# line 704 "lua.stx"
{ yyval.vInt = yypvt[-1].vInt; } break;
case 81:
# line 707 "lua.stx"
{yyval.vInt=1;} break;
case 82:
# line 709 "lua.stx"
{
		  yyval.vInt=yypvt[-2].vInt+1;
		  if (yyval.vInt%FIELDS_PER_FLUSH == 0) flush_record(FIELDS_PER_FLUSH);
		} break;
case 83:
# line 716 "lua.stx"
{ 
	       push_field(luaI_findconstant(yypvt[-2].pNode));
	      } break;
case 84:
# line 721 "lua.stx"
{ yyval.vInt = 0; } break;
case 85:
# line 722 "lua.stx"
{ yyval.vInt = yypvt[-1].vInt; } break;
case 86:
# line 725 "lua.stx"
{yyval.vInt=1;} break;
case 87:
# line 727 "lua.stx"
{
		  yyval.vInt=yypvt[-2].vInt+1;
		  if (yyval.vInt%FIELDS_PER_FLUSH == 0) 
		    flush_list(yyval.vInt/FIELDS_PER_FLUSH - 1, FIELDS_PER_FLUSH);
		} break;
case 88:
# line 735 "lua.stx"
{
	   nvarbuffer = 0; 
	   add_varbuffer(yypvt[-0].vLong);
	   yyval.vInt = (yypvt[-0].vLong == 0) ? 1 : 0;
	  } break;
case 89:
# line 741 "lua.stx"
{ 
	   add_varbuffer(yypvt[-0].vLong);
	   yyval.vInt = (yypvt[-0].vLong == 0) ? yypvt[-2].vInt + 1 : yypvt[-2].vInt;
	  } break;
case 90:
# line 747 "lua.stx"
{ yyval.vLong = yypvt[-0].vLong; } break;
case 91:
# line 749 "lua.stx"
{
	   yyval.vLong = 0;		/* indexed variable */
	  } break;
case 92:
# line 753 "lua.stx"
{
	   code_byte(PUSHSTRING);
	   code_word(luaI_findconstant(yypvt[-0].pNode));
	   yyval.vLong = 0;		/* indexed variable */
	  } break;
case 93:
# line 761 "lua.stx"
{
	   Word s = luaI_findsymbol(yypvt[-0].pNode);
	   int local = lua_localname (s);
	   if (local == -1)	/* global var */
	    yyval.vLong = s + 1;		/* return positive value */
           else
	    yyval.vLong = -(local+1);		/* return negative value */
	  } break;
case 94:
# line 771 "lua.stx"
{ lua_pushvar(yypvt[-0].vLong); } break;
case 95:
# line 774 "lua.stx"
{store_localvar(luaI_findsymbol(yypvt[-0].pNode), 0); yyval.vInt = 1;} break;
case 96:
# line 776 "lua.stx"
{
	     store_localvar(luaI_findsymbol(yypvt[-0].pNode), yypvt[-2].vInt);
	     yyval.vInt = yypvt[-2].vInt+1;
	    } break;
case 97:
# line 782 "lua.stx"
{ yyval.vInt = 0; } break;
case 98:
# line 783 "lua.stx"
{ yyval.vInt = yypvt[-0].vLong; } break;
case 99:
# line 786 "lua.stx"
{ lua_debug = yypvt[-0].vInt; } break;
	}
	goto yystack;		/* reset registers in driver code */
}
